package com.simibubi.create.content.contraptions.relays.advanced.sequencer;

import java.util.Random;

import com.simibubi.create.AllItems;
import com.simibubi.create.AllTileEntities;
import com.simibubi.create.content.contraptions.base.HorizontalAxisKineticBlock;
import com.simibubi.create.content.contraptions.base.IRotate;
import com.simibubi.create.content.contraptions.base.KineticBlock;
import com.simibubi.create.foundation.block.ITE;
import com.simibubi.create.foundation.gui.ScreenOpener;

import net.minecraft.client.player.LocalPlayer;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.core.Direction.Axis;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.context.BlockPlaceContext;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition.Builder;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.IntegerProperty;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.fml.DistExecutor;

public class SequencedGearshiftBlock extends HorizontalAxisKineticBlock implements ITE<SequencedGearshiftTileEntity> {

	public static final BooleanProperty VERTICAL = BooleanProperty.create("vertical");
	public static final BooleanProperty POSITIVE = BooleanProperty.create("positive");
	public static final IntegerProperty STATE = IntegerProperty.create("state", 0, 5);

	public SequencedGearshiftBlock(Properties properties) {
		super(properties);
	}

	@Override
	protected void createBlockStateDefinition(Builder<Block, BlockState> builder) {
		super.createBlockStateDefinition(builder.add(STATE, VERTICAL, POSITIVE));
	}

	@Override
	public boolean shouldCheckWeakPower(BlockState state, LevelReader world, BlockPos pos, Direction side) {
		return false;
	}

	@Override
	public void neighborChanged(BlockState state, Level worldIn, BlockPos pos, Block blockIn, BlockPos fromPos,
		boolean isMoving) {
		if (worldIn.isClientSide)
			return;
		if (!worldIn.getBlockTicks().willTickThisTick(pos, this))
			worldIn.scheduleTick(pos, this, 0);
	}

	@Override
	public void tick(BlockState state, ServerLevel worldIn, BlockPos pos, Random r) {
		boolean previouslyPowered = state.getValue(STATE) != 0;
		boolean isPowered = worldIn.hasNeighborSignal(pos);
		withTileEntityDo(worldIn, pos, sgte -> sgte.onRedstoneUpdate(isPowered, previouslyPowered));
	}

	@Override
	public boolean hasShaftTowards(BlockState state, Direction face) {
		if (state.getValue(VERTICAL))
			return face.getAxis().isVertical();
		return super.hasShaftTowards(state, face);
	}

	@Override
	public InteractionResult use(BlockState state, Level worldIn, BlockPos pos, Player player, InteractionHand handIn,
		BlockHitResult hit) {
		ItemStack held = player.getMainHandItem();
		if (AllItems.WRENCH.isIn(held))
			return InteractionResult.PASS;
		if (held.getItem() instanceof BlockItem blockItem) {
			if (blockItem.getBlock() instanceof KineticBlock && hasShaftTowards(state, hit.getDirection(), worldIn, hit.getBlockPos()))
				return InteractionResult.PASS;
		}

		DistExecutor.unsafeRunWhenOn(Dist.CLIENT,
			() -> () -> withTileEntityDo(worldIn, pos, te -> this.displayScreen(te, player)));
		return InteractionResult.SUCCESS;
	}

	@OnlyIn(value = Dist.CLIENT)
	protected void displayScreen(SequencedGearshiftTileEntity te, Player player) {
		if (player instanceof LocalPlayer)
			ScreenOpener.open(new SequencedGearshiftScreen(te));
	}

	private static boolean sideFacesShaft(BlockPlaceContext context, Direction side) {
		BlockPos pos = context.getClickedPos().relative(side);
		BlockState state = context.getLevel().getBlockState(pos);
		return state.getBlock() instanceof IRotate ir
				&& ir.hasShaftTowards(state, side.getOpposite(), context.getLevel(), pos);
	}

	@Override
	public BlockState getStateForPlacement(BlockPlaceContext context) {
		Direction preferredFacing = null;
		for (Direction dir : context.getNearestLookingDirections()) {
			if (sideFacesShaft(context, dir.getOpposite())) {
				preferredFacing = dir;
				break;
			}
		}

		if (preferredFacing != null && (context.getPlayer() == null || !context.getPlayer().isShiftKeyDown()))
			return withFacing(preferredFacing, context);
		return withFacing(context.getNearestLookingDirection(), context);
	}

	private BlockState withFacing(Direction dir, BlockPlaceContext context) {
		Axis axis = dir.getAxis();
		BlockState state = defaultBlockState()
				.setValue(VERTICAL, axis.isVertical())
				.setValue(POSITIVE, dir.getAxisDirection() == Direction.AxisDirection.POSITIVE);
		if (axis.isVertical())
			return state.setValue(HORIZONTAL_AXIS, context.getHorizontalDirection().getAxis());
		return state.setValue(HORIZONTAL_AXIS, axis);
	}

	public Direction getFacing(BlockState state) {
		Direction.AxisDirection axisDir = state.getValue(POSITIVE)
				? Direction.AxisDirection.POSITIVE
				: Direction.AxisDirection.NEGATIVE;
		return Direction.fromAxisAndDirection(getRotationAxis(state), axisDir);
	}

	@Override
	public InteractionResult onWrenched(BlockState state, UseOnContext context) {
		BlockState newState = state;

		if (context.getClickedFace().getAxis() != Axis.Y)
			if (newState.getValue(HORIZONTAL_AXIS) != context.getClickedFace().getAxis())
				newState = newState.cycle(VERTICAL);

		return super.onWrenched(newState, context);
	}

	@Override
	public Axis getRotationAxis(BlockState state) {
		if (state.getValue(VERTICAL))
			return Axis.Y;
		return super.getRotationAxis(state);
	}

	@Override
	public Class<SequencedGearshiftTileEntity> getTileEntityClass() {
		return SequencedGearshiftTileEntity.class;
	}

	@Override
	public BlockEntityType<? extends SequencedGearshiftTileEntity> getTileEntityType() {
		return AllTileEntities.SEQUENCED_GEARSHIFT.get();
	}

	@Override
	public boolean hasAnalogOutputSignal(BlockState p_149740_1_) {
		return true;
	}

	@Override
	public int getAnalogOutputSignal(BlockState state, Level world, BlockPos pos) {
		return state.getValue(STATE);
	}

}
